package javaspacesdatenstrukturen;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import net.jini.space.*;
import net.jini.core.lease.*;

/**
 * Title:        Verteilte Datenstrukturen
 * Description:  Building Blocks fr verteilte Anwendungen mit JavaSpaces
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class MP3Requester extends Applet
        implements ActionListener, Runnable
{
    private JavaSpace space;
    private Thread resultTaker;
    private String from;        // eindeutiger Name fr die MP3 verlangende Person

    // GUI
    private String newline = System.getProperty("line.separator");
    private Label label1;
    private Label label2;
    private Label label3;
    private Label statusLabel;
    private Label mp3Label;
    private TextField userTextField;
    private TextField fileTextField;
    private Button encodeButton;
    private TextArea statusOutput;
    private List mp3Links;

    public void init() {
        space = SpaceAccessor.getSpace();

        // Thread zum Verteilen und Sammeln der Ergebnisse
        if (resultTaker == null) {
            resultTaker = new Thread(this);
            resultTaker.start();
        }

        // GUI Setup
    	setLayout(null);
    	setSize(600,500);

        // Label
        label1 = new Label("Geben Sie einen eindeutigen Namen ein, z.B. die email Adresse:", Label.LEFT);
        label1.setBounds(80,12,600,25);
        add(label1);

        // Benutzername
    	userTextField = new TextField();
    	userTextField.setBounds(80,37,310,24);
    	add(userTextField);

    	// Labels
    	label2 = new Label("Absoluter Dateiname der WAV Datei:", Label.LEFT);
    	label2.setBounds(80,70,600,25);
    	add(label2);
    	label3 = new Label("(Beispiel, C:\\Temp\\MyMusic\\test.wav)", Label.LEFT);
    	label3.setBounds(80,90,600,25);
    	add(label3);

    	// Textfeld fr den Dateinamen
    	fileTextField = new TextField();
    	fileTextField.setBounds(80,115,310,24);
    	add(fileTextField);

    	// Encode Button
    	encodeButton = new java.awt.Button();
    	encodeButton.setLabel("Encode WAV!");
    	encodeButton.setBounds(395,115,90,24);
    	encodeButton.addActionListener(this);
    	add(encodeButton);

    	// Status Label
        statusLabel = new Label("Status:",Label.RIGHT);
        statusLabel.setBounds(0,170,75,21);
        add(statusLabel);

        // Status Zeile
        statusOutput = new TextArea();
        statusOutput.setBounds(80,170,500,100);
        add(statusOutput);

        // Ergebnislabel
        mp3Label = new Label("Resultate:",Label.RIGHT);
        mp3Label.setBounds(0,290,75,21);
        add(mp3Label);

        //// Text Area fr Resultate
        //mp3Output = new TextArea();
        //mp3Output.setBounds(80,290,500,300);
        //add(mp3Output);

        // Liste mit den MP3 Dateinamen
        mp3Links = new List(10, false);
        mp3Links.setBounds(80,290,500,150);
        add(mp3Links);
    }

    // "Encode" wurde gedrckt
    public void actionPerformed(ActionEvent event) {

        from = userTextField.getText();
        if (from.equals("")) {
            statusOutput.append("Geben Sie einen eindeutigen Benutzernamen ein." + newline);
            return;
        }

        String inputName = fileTextField.getText();
        if (inputName.equals("") || (!inputName.endsWith(".wav"))) {
            statusOutput.append("Geben Sie den absoluten Namen der WAV Datei ein." + newline);
            return;
        }
        fileTextField.setText("");

        // Daten des WAV Files in eine ENtry einpacken
        byte[] rawData = null;

        rawData = Utils.getRawData(inputName);

        // einen MP3 Request generieren
        if (rawData != null) {
            append("MP3 Request", inputName, rawData, from);
        }
    }

    private void append(String channel, String inputName,
            byte[] rawData, String from) {
        statusOutput.append("Suche \"" + channel + "\"..." + newline);
        Integer num = getRequestNumber(channel);
        MP3Request request =
            new MP3Request(channel, num, inputName, rawData, from);

        statusOutput.append("Sende Request " + num +
            " an \"" + channel + "\"..." + newline);
        try {
            space.write(request, null, Lease.FOREVER);
        } catch (Exception e) {
            statusOutput.append("Fehler beim Schreiben in den Channel." + newline);
            e.printStackTrace();
            return;
        }
        statusOutput.append("Anfrage " + num +
            " an \"" + channel + "senden \"... Erledigt." + newline);
    }


    private Integer getRequestNumber(String channel) {
        try {
            Index template = new Index("tail", channel);
            Index tail = (Index) space.take(template, null, Long.MAX_VALUE);
            tail.increment();
            space.write(tail, null, Lease.FOREVER);
            return tail.getPosition();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }


    // Thread zeigt MP3 Links an
    public void run() {
        MP3Result template = new MP3Result(from);
        MP3Result result = null;
        String outputName = "";   // Name der MP3 Dateien

        while(true) {
            try {
                result = (MP3Result)
                    space.take(template, null, Long.MAX_VALUE);
                statusOutput.append("Ergebnis fuer: " + result.inputName + "." + newline);
                int pos = result.inputName.indexOf(".wav");
                outputName = result.inputName.substring(0, pos) + ".mp3";
                Utils.putRawData(result.data, outputName);
                displayResult(outputName);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    private void displayResult(String outputName) {
        statusOutput.append("Datei " + outputName + " wurde geschrieben." + newline);
        mp3Links.addItem(outputName);
    }


   public boolean action(Event event, Object arg){
        if (event.target == mp3Links){
             String fileName = (String)arg;
             System.out.println("Action - mit Datei: " + fileName);

             try {
                String[] cmdArray = { "\"C:\\Programme\\Winamp\\Winamp\"", fileName};
                Process subProcess = Runtime.getRuntime().exec(cmdArray);
             } catch (Exception e) {
                System.out.println("Fehler beim Apspielen der Datei.");
                e.printStackTrace();
                return false;
             }
         }
        return true;
    }

}