package javaspacesdatenstrukturen;

import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;
import java.io.*;
import java.lang.Runtime;

/**
 * Title:        Verteilte Datenstrukturen
 * Description:  Building Blocks fr verteilte Anwendungen mit JavaSpaces
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class MP3Worker {
    private JavaSpace space;
    private String channel;

    public static void main(String[] args) {
        System.out.println("[MP3Worker.main()]Start");
        MP3Worker worker = new MP3Worker();
        System.out.println("[MP3Worker.main()]startWork()");
        worker.startWork();
        System.out.println("[MP3Worker.main()]Ende");
    }

    public void startWork() {
        System.out.println("[MP3Worker.startWork()]");
        space = SpaceAccessor.getSpace();
        channel = "MP3 Request";

        System.out.println("[MP3Worker.startWork()]processNextRequest()");
        while(true) {
            processNextRequest();
        }
    }

    private void processNextRequest() {
        System.out.println("[MP3Worker.processNextRequest()]");
        Index tail = readIndex("tail", channel);
        Index head = removeIndex("head", channel);

        if (tail.position.intValue() < head.position.intValue()) {
            // es liegen keine Anfragen vor
            writeIndex(head);
            return;
        }

        // neue Anfrage
        MP3Request request = removeRequest(channel, head.position);
        head.increment();
        writeIndex(head);

        if (request == null) {
            System.out.println("[MP3Worker.processNextRequest()]Kommunikationsfehler.");
            return;
        }

        // MP3 Request, rufe BladeEnc Programm auf
        // Konvertiere WAV in MP3
        System.out.println("[MP3Worker.processNextRequest()]Anfrage von " + request.from + " fuer: " + request.inputName);

        String inputName = request.inputName;
        String from = request.from;
        byte[] inputData = request.data;
        byte[] outputData = null;
        String tmpInputFile = "./tmp" + request.position + ".wav";
        String tmpOutputFile = "./tmp" + request.position + ".mp3";
        Process subProcess = null;

        Utils.putRawData(inputData, tmpInputFile);
        try {
            String[] cmdArray = { "\"C:\\Programme\\BladeEnc\\BladeEnc\"", "-quit", "-quiet", "-progress=0", tmpInputFile, tmpOutputFile};
            subProcess = Runtime.getRuntime().exec(cmdArray);
            subProcess.waitFor();
       	} catch (Exception e) {
            System.out.println("[MP3Worker.processNextRequest()]Fehler beim Encoding.");
            e.printStackTrace();
            return;
    	}

    	System.out.println("[MP3Worker.processNextRequest()]Prozess wurde beendet mit Programmcode " + subProcess.exitValue());
       	if (subProcess.exitValue() != 0) {
       	    System.out.println("[MP3Worker.processNextRequest()]Fehler beim Kodieren.");
       	    return;
       	}

    	// MP3 in den Space stellen
        outputData = Utils.getRawData(tmpOutputFile);
        MP3Result result = new MP3Result(inputName, outputData, from);
        try {
            space.write(result, null, Lease.FOREVER);
            System.out.println("[MP3Worker.processNextRequest()]MP3 fr " + inputName + " steht im Space.");

    	    // delete the temporary files
    	    new File(tmpInputFile).delete();
    	    new File(tmpOutputFile).delete();
        } catch (Exception e) {
            System.out.println("[MP3Worker.processNextRequest()]Fehler beim Schreiben in den Channel.");
            e.printStackTrace();
            return;
        }
    }

    private MP3Request removeRequest(String channel,
        Integer position)
    {
        System.out.println("[MP3Worker.removeRequest()]");
        MP3Request template = new MP3Request(channel, position);
        MP3Request request = null;
        try {
            request = (MP3Request)
                space.take(template, null, Long.MAX_VALUE);
            return request;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // entferne Indes ("tail", "header") aus dem Channel
    private Index removeIndex(String type, String channel) {
        System.out.println("[MP3Worker.removeIndex()]");
        Index template = new Index(type, channel);
        Index index = null;
        try {
            return (Index)space.take(template, null, Long.MAX_VALUE);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // lies Index ("head" or "tail") aus dem Channel
    private Index readIndex(String type, String channel) {
        System.out.println("[MP3Worker.readIndex()]");
        Index template = new Index(type, channel);
        Index index = null;
        try {
            return (Index)space.read(template, null, Long.MAX_VALUE);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // schreibe head oder tail in den Channel
    private void writeIndex(Index index) {
        System.out.println("[MP3Worker.writeIndex()]");
        try {
            space.write(index, null, Lease.FOREVER);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
