package javaspacesfehlertoleranz;

import java.rmi.RemoteException;

import net.jini.core.entry.*;
import net.jini.core.lease.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.space.JavaSpace;
import com.sun.jini.mahalo.TxnManagerImpl_Stub;
/**
 * Title:        Fehlertoleranz in JavaSpaces Applikationen
 * Description:  Verteilter Rechenr mit Transaktionen zur Erhhung der Fehlertoleranz
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Master {
    private JavaSpace space;
    private TransactionManager mgr;


    public static void main(String[] args) {
        System.out.println("[Master.main()]Start");
        Master master = new Master();
        master.startComputing();
        System.out.println("[Master.main()]Ende");
    }


    private void startComputing() {
        System.out.println("[Master.startComputing()]");

        space = SpaceAccessor.getSpace();
        mgr = TransactionManagerAccessor.getManager();

        System.out.println("[Master.startComputing()]generateTasks()");
        generateTasks();
        collectResults();
        System.out.println("[Master.startComputing()]collectResults()");
    }


    private void generateTasks() {
        System.out.println("[Master.generateTask()]");

        boolean written;

        for (int i = 0; i < 10; i++) {
            written = false;
            while (!written) {
                System.out.println("[Master.generateTask()]AddTask() "+i+" ins Space schreiben");
                written = writeTask(new AddTask(new Integer(i), new Integer(i)));
            }
        }
        for (int i = 0; i < 10; i++) {
            written = false;
            while (!written) {
                System.out.println("[Master.generateTask()]MultTask() "+i+" ins Space schreiben");
                written = writeTask(new MultTask(new Integer(i), new Integer(i)));
            }
        }
    }


    private void collectResults() {
        System.out.println("[Master.collectResults()]");

        ResultEntry result;
        for (int i = 0; i < 20; i++) {
            result = null;
            while (result == null) {
                System.out.println("[Master.collectResults()]takeResult() "+i+" dem Space entnehmen");
                result = takeResult();
            }
            System.out.println(result);
        }
    }


    private boolean writeTask(Command task) {
        System.out.println("[Master.writeTask()]");

        // Transaktion mit il Minuten Leasezeit
        Transaction txn = null;
        // Leasingdauer : il Minuten
        int il=3;
        txn=getTransaction(1000*il*60);
        System.out.println("[Master.writeTask()]Transaktion gestartet");
        if (txn == null) {
            throw new RuntimeException("[Master.writeTask()]Kann keine Transaktion erhalten");
        }

        try {
            try {
                space.write(task, txn, Lease.FOREVER);
                //space.write(task, null, Lease.FOREVER);
            } catch (Exception e) {
                System.err.println("[Master.writeTask()]Transaktion schlug fehl");
                txn.abort();
                e.printStackTrace();
                return false;
            }
            txn.commit();
            System.out.println("[Master.writeTask()]Transaktion wurde commited");

            return true;
        } catch (Exception e) {
            System.err.println("[Master.writeTask()]Transaktion schlug fehl");
            return false;
        }
    }


   protected ResultEntry takeResult() {
        System.out.println("[Master.takeResult()]");

        // ik Minuten Lease
        int ik=3;
        Transaction txn = getTransaction(1000*ik*60);
        System.out.println("[Master.takeResult()]Transaktion gestartet");
        if (txn == null) {
            throw new RuntimeException("[Master.takeResult()]Transaktion wurde nicht korrekt bestimmt");
        }

        ResultEntry template = new ResultEntry();
        ResultEntry result;
        try {
            try {
                //result = (ResultEntry)space.take(template, txn, Long.MAX_VALUE);
                result = (ResultEntry)space.take(template, null, Long.MAX_VALUE);
            } catch (Exception e) {
                txn.abort();
                System.err.println("[Master.takeResult()]Transaktion schlug fehl");
                return null;
            }
            txn.commit();
            System.out.println("[Master.takeResult()]Transaktion wurde committed");
            return result;
        } catch (Exception e) {
            System.err.println("[Master.takeResult()]Transaktion schlug fehl");
            return null;
        }
    }


    public Transaction getTransaction(long leaseTime) {
        System.out.println("[Master.getTransaction()]");

        try {
            Transaction.Created created=TransactionFactory.create(mgr, leaseTime);
            return created.transaction;
        } catch(RemoteException e) {
            e.printStackTrace();
            return null;
        } catch(LeaseDeniedException e) {
            e.printStackTrace();
            return null;
        }
    }
}

