package javaspacesfehlertoleranz;

import java.rmi.RemoteException;

import net.jini.core.entry.*;
import net.jini.core.lease.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.space.JavaSpace;

/**
 * Title:        Fehlertoleranz in JavaSpaces Applikationen
 * Description:  Verteilter Rechenr mit Transaktionen zur ERhhung der Fehlertoleranz
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Worker {
    private JavaSpace space;
    private TransactionManager mgr;

    public static void main(String[] args) {
        System.out.println("[Worker.main()]Start");
        Worker worker = new Worker();
        worker.startWork();
        System.out.println("[Worker.main()]Ende");
    }

    public Worker() {
        System.out.println("[Worker.Worker()]");
        space = SpaceAccessor.getSpace();
        mgr = TransactionManagerAccessor.getManager();
    }


    public void startWork() {
        System.out.println("[Worker.startWork()]");

        TaskEntry template = new TaskEntry();

        for (;;) {

            // Transaktion mit 10 Minuten Leasezeit
            System.out.println("[Worker.startWork()]Transaktion wird gestartet");
            Transaction txn = getTransaction(1000*10*60);
            if (txn == null) {
                throw new RuntimeException("[Worker.startWork()]Transaktion konnte nicht erhalten werden");
            }

            try {
                try {
                    // Aufgabe der Transaktion
                    System.out.println("[Worker.startWork()]Entnehmen einer Aufgabe aus dem Space");
                    TaskEntry task = (TaskEntry)
                        space.take(template, txn, Long.MAX_VALUE);

                    // Aufgabe ausfhren
                    System.out.println("[Worker.startWork()]Aufgabe ausfuehren");
                    Entry result = task.execute();

                    // Ergebnis in den Space zurck schreiben
                    System.out.println("[Worker.startWork()]Ergebnis ins Space schreiben");
                    if (result != null) {
                        space.write(result, txn, 1000*60*10);
                    }
                } catch (Exception e) {
                    System.out.println("[Worker.startWork()]Transaktion gecancelled:" + e);
                    txn.abort();
                }
                txn.commit();
                System.out.println("[Worker.startWork()]Transaktion erfolgreich abgeschlossen");
            } catch (Exception e) {
                System.out.println("[Worker.startWork()]Transaktion schlug fehl:" + e);
            }
        }
    }



    public Transaction getTransaction(long leaseTime) {
        System.out.println("[Worker.getTransaction()]");

        try {
            Transaction.Created created =
                TransactionFactory.create(mgr, leaseTime);
            return created.transaction;
        } catch(RemoteException e) {
            e.printStackTrace();
            return null;
        } catch(LeaseDeniedException e) {
            e.printStackTrace();
            return null;
        }
    }
}
