package javaspacesmandelbrot;

import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;

/**
 * Title:        Mandelbrot Set als JavaSpaces Anwendung
 * Description:  Einfaches Beispiel fr den Master-Slave Aufbau einer JavaSpaces Anwendung
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Worker {
    public static void main(String[] args) {
        JavaSpace space = SpaceAccessor.getSpace();

        Task template = new Task();
        Task task;

        for (;;) {
            byte[][] points;
            try {
                task = (Task)
                    space.take(template, null, Long.MAX_VALUE);
                System.out.println("Task uebernimmt Aufgabe "
                    + task.start +
                    " fuer Job " + task.jobId);

                points = calculateMandelbrot(task);
                Result result = new Result(task.jobId,
                    task.start, points);

                System.out.println(
                    "Task schreibt Ergebnisse fuer die Aufgabe " +
                    result.start + " fuer Job " + result.jobId);
                space.write(result, null, Lease.FOREVER);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static byte[][] calculateMandelbrot(Task task) {
        double x, y, xx, a, b;
        int start = task.start.intValue();
        int lines = task.lines.intValue();
        int end = start + task.lines.intValue();
        int width = task.width.intValue();
        int height = task.height.intValue();

        double da = task.x2.doubleValue()/width;
        double db = task.y2.doubleValue()/height;

        b = task.y1.doubleValue();

        byte[][] points = new byte[width][lines];

        for (int i = 0; i < start; i++) {
            b = b + db;
        }

        int k = 0;

        for (int i = start; i < end; i++, k++) {
            a = task.x1.doubleValue();
            for (int j = 0; j < width; j++) {
                byte n = 0;
                x = 0.0;
                y = 0.0;
                while ( (n < 100) && ( (x*x)+(y*y) < 4.0) ) {
                    xx = x * x - y * y + a;
                    y = 2 * x * y + b;
                    x = xx;
                    n++;
                }
                points[j][k] = n;
                a = a + da;
            }
            b = b + db;
        }
        return points;
    }
}
