package javaspacesskalierbarkeit;

import java.rmi.RemoteException;

import net.jini.core.entry.*;
import net.jini.core.lease.*;
import net.jini.core.lookup.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.space.JavaSpace;

/**
 * Title:        Skalierbarkeit von JavaSpaces Lsungen
 * Description:  Verteilter Rechenr, der ber mehere JavaSpaces die Arbeit verteiltund wieder einsammelt
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Master {
    private ServiceItem[] spaceServices = null;
    private int numSpaces = 0;
    private TransactionManager mgr;

    public static void main(String[] args) {
        System.out.println("[Worker.main()]Start");
        Master master = new Master();
        System.out.println("[Worker.main()]startComputing");
        master.startComputing();
        System.out.println("[Worker.main()]Ende");
    }

    private void startComputing() {
        System.out.println("[Worker.startComputing()]");
        ServiceMatches serviceMatches = SpaceAccessor.getSpaces();
        if (serviceMatches == null) {
            System.out.println("[Worker.startComputing()]Es wurden keine Spaces gefunden... Das Programm wird abgebrochen.");
        } else {
            spaceServices = serviceMatches.items;
            numSpaces = serviceMatches.totalMatches;
            System.out.println("[Worker.startComputing()]Es wurden " + numSpaces + " Spaces gefunden");
            mgr = TransactionManagerAccessor.getManager();

            System.out.println("[Worker.startComputing()]generateTask()");
            generateTasks();
            System.out.println("[Worker.startComputing()]collectResult()");
            collectResults();
        }
    }


    private void generateTasks() {
        System.out.println("[Worker.generateTask()]");
        boolean written;
        ServiceItem spaceService;
        JavaSpace space;
        ServiceID id;
        Integer num;

        for (int i = 0; i < 10; i++) {
            spaceService = spaceServices[i % numSpaces];
            space = (JavaSpace)spaceService.service;
            id = spaceService.serviceID;
            num = new Integer(i);

            written = false;
            while (!written) {
                written = writeTask(space, new AddTask(num, num));
            }
            System.out.println("[Worker.generateTask()]Die Additionsaufgabe " + i + " wurde in Space " + id+" eingefuegt");

            written = false;
            while (!written) {
                written = writeTask(space, new MultTask(num, num));
            }
            System.out.println("[Worker.generateTask()]Die Multipliikationsaufgabe " + i + " wurde in Space " + id+" eingefuegt");
        }
    }



    private void collectResults() {
        System.out.println("[Worker.collectResults()]");
        ResultEntry result;
        for (int i = 0; i < 10; i++) {
            // bestimme zwei Ergebnisse: eine Addition, eine Multiplikation
            for (int j = 0; j < 2; j++) {
                result = null;
                while (result == null) {
                    result = takeResult(i);
                }
            }
        }
    }

    private boolean writeTask(JavaSpace space, Command task) {
        System.out.println("[Worker.writeTask()]");
        // Transaktion mit 10 Minuten Lease
        Transaction txn = getTransaction(1000*10*60);
        if (txn == null) {
            throw new RuntimeException("[Worker.writeTask()]Transaktion konnte nicht generiert werden");
        }
        System.out.println("[Worker.writeTask()]Transaktion wurde gestartet");

        try {
            try {
                //space.write(task, txn, Lease.FOREVER);
                space.write(task, null, Lease.FOREVER);
            } catch (Exception e) {
                txn.abort();
                System.out.println("[Worker.writeTask()]Transaktion wurde abgebrochen");
                return false;
            }
            txn.commit();
            System.out.println("[Worker.writeTask()]Transaktion wurde committed");
            return true;
        } catch (Exception e) {
            System.err.println("[Worker.writeTask()]Transaktion schlug fehl");
            return false;
        }
    }



    protected ResultEntry takeResult(int i) {
        System.out.println("[Worker.takeResult()]");

        // try to get a transaction with a 10-min lease time
        Transaction txn = getTransaction(1000*10*60);
        if (txn == null) {
            throw new RuntimeException("Es konnte keine Transaktion bestimmt werden");
        }
        System.out.println("[Worker.takeResult()]Transaktion wurde gestartet");

        ServiceItem spaceService = spaceServices[i % numSpaces];
        JavaSpace space = (JavaSpace)spaceService.service;
        ServiceID id = spaceService.serviceID;
        ResultEntry template = new ResultEntry(i);
        ResultEntry result;

        try {
            try {
                result = (ResultEntry)
                    //space.take(template, txn, Long.MAX_VALUE);
                    space.take(template, null, Long.MAX_VALUE);
            } catch (Exception e) {
                txn.abort();
                System.out.println("[Worker.takeResult()]Transaktion wurde abgebrochen");
                return null;
            }
            txn.commit();
            System.out.println("[Worker.takeResult()]Transaktion wurde committed");
            if (AddResult.class.isInstance(result)) {
                System.out.println("Lesen der Addition " + i + " aus Space " + id);
            } else {
                System.out.println("Lesen der Multiplikation " + i + " aus Space " + id);
            }
            return result;
        } catch (Exception e) {
            System.err.println("Die Transaktion schlug fehl");
            return null;
        }
    }

    private Transaction getTransaction(long leaseTime) {
        System.out.println("[Worker.getTransaction()]");
        try {
            Transaction.Created created =
                TransactionFactory.create(mgr, leaseTime);
            return created.transaction;
        } catch(RemoteException e) {
            e.printStackTrace();
            return null;
        } catch(LeaseDeniedException e) {
            e.printStackTrace();
            return null;
        }
    }
}

