package javaspacesskalierbarkeit;

import java.rmi.RemoteException;

import net.jini.core.entry.*;
import net.jini.core.lease.*;
import net.jini.core.lookup.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.space.JavaSpace;

/**
 * Title:        Skalierbarkeit von JavaSpaces Lsungen
 * Description:  Verteilter Rechenr, der ber mehere JavaSpaces die Arbeit verteiltund wieder einsammelt
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Worker {
    private ServiceItem[] spaceServices = null;
    private int numSpaces = 0;
    private TransactionManager mgr;

    public static void main(String[] args) {
        System.out.println("[Worker.main()]Start");
        Worker worker = new Worker();
        System.out.println("[Worker.main()]Ende");
    }

    public Worker() {
        System.out.println("[Worker.Worker()]");
        ServiceMatches serviceMatches = SpaceAccessor.getSpaces();
        if (serviceMatches == null) {
            System.out.println("[Worker.Worker()]Es wurden keine Spaces gefunden... Das Programm wird abgebrochen.");
        } else {
            spaceServices = serviceMatches.items;
            numSpaces = serviceMatches.totalMatches;
            if (numSpaces==1) System.out.println("[Worker.Worker()]Es wurde " + numSpaces + " Spaces gefunden.");
            else System.out.println("[Worker.Worker()]Es wurden " + numSpaces + " Spaces gefunden.");

            mgr = TransactionManagerAccessor.getManager();
            System.out.println("[Worker.Worker()]startWork()");
            startWork();
        }
    }


    public void startWork() {
        System.out.println("[Worker.startWork()]");
        ServiceItem spaceService;
        JavaSpace space;
        ServiceID id;

        TaskEntry template = new TaskEntry();

        for (int i=0; ; i++) {
            spaceService = spaceServices[i % numSpaces];
            space = (JavaSpace)spaceService.service;
            id = spaceService.serviceID;


            // Transaktion mit 10 Min Leaseingdauer
            Transaction txn = getTransaction(1000*10*60);
            if (txn == null) {
                throw new RuntimeException("Transaktion konnte nicht generiert werden");
            }
            System.out.println("[Worker.startWork()]Transaktion wurde gestartet");

            try {
                try {
                    // Task gehrt zur Transaktion
                    TaskEntry task = (TaskEntry)
                        space.take(template, txn, Long.MAX_VALUE);

                    // Ausfhren der Task
                    Entry result = task.execute();

                    if (AddTask.class.isInstance(task)) {
                        System.out.println("[Worker.startWork()]Additionsaufgabe " + ((AddTask)task).a +
                                " aus Space " + id + "...wird bearbeitet...");
                    } else {
                        System.out.println("[Worker.startWork()]Multiplikationsaufgabe " + ((MultTask)task).a +
                                " aus Space " + id + "...wird bearbeitet...");
                    }

                    // schreiben der Ergebnisse in den Space
                    if (result != null) {
                        space.write(result, txn, 1000*60*10);
                        System.out.println("Schreibe das Ergebnis in Space " + id);
                    }
                } catch (Exception e) {
                    System.out.println("[Worker.startWork()]Task wurde gecancelled:" + e);
                    txn.abort();
                    System.out.println("[Worker.startWork()]Transaktion wurde abgebrochen");
                }
                txn.commit();
                System.out.println("[Worker.startWork()]Transaktion wurde committed");
            } catch (Exception e) {
                System.out.println("[Worker.startWork()]Transaktion schlug fehl:" + e);
            }
        }
    }



    public Transaction getTransaction(long leaseTime) {
        System.out.println("[Worker.getTransaction()");
        try {
            Transaction.Created created =
                TransactionFactory.create(mgr, leaseTime);
            return created.transaction;
        } catch(RemoteException e) {
            e.printStackTrace();
            return null;
        } catch(LeaseDeniedException e) {
            e.printStackTrace();
            return null;
        }
    }
}
