package javaspacestransaktionen;

import java.io.*;

import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;

/**
 * Title:        JavaSpaces Transaktionen
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class SpaceRelay {
    private String sourceName;
    private JavaSpace sourceSpace;
    private JavaSpace[] targetSpaces;
    private String[] targetNames;
    private int numMessages;
    private int k=0;


    /* schreibe numMessages Message Entries in den Source Space
     */
    private void createMessages() {
        for (int i = 0; i < numMessages; i++) {
            Message msg = new Message();
            msg.content = "" + i;
            try {
                sourceSpace.write(msg, null, Lease.FOREVER);
                System.out.println("[SpaceRelay.createMessages()]Schreibe Message " + i + " in " + sourceName);
            } catch (Exception e) {
                System.err.println("[SpaceRelay.createMessages()]Fehler beim Schreiben der " + i + "'ten Message: " + e);
            }
        }
    }


    /* entnimm numMessages Message Entries aus dem Source Space und schreibe
     * die Entries in jeden der Target Spaces
     */
    private void relayMessages() {
        TransactionManager mgr = TransactionManagerAccessor.getManager();
        Message template = new Message();
        Message msg;

        for (int i = 0; i < numMessages; i++) {
            Transaction.Created trc = null;
            try {
                System.out.println("[SpaceRelay.relayMessages()]Kreieren der Transaktion " + i);
                trc = TransactionFactory.create(mgr, 300000);
            } catch (Exception e) {
                System.err.println("[SpaceRelay.relayMessages()]Fehler beim Kreieren der Transaktion " + e);
                return;
            }
            Transaction txn = trc.transaction;

            try {
                try {
                    template.content = "" + i;

                    // lies die Message der Transaktion
                    //msg=(Message)sourceSpace.read(template,null,Long.MAX_VALUE);
                    //System.out.println("[SpaceRelay.relayMessages()]Message " + i + " wurde aus " + sourceName+" gelesen");
                    msg = (Message)sourceSpace.take(template, txn, Long.MAX_VALUE);
                    System.out.println("[SpaceRelay.relayMessages()]Message " + i + " wurde aus " + sourceName+" entnommen");

                    // schreibe die Message in die anderen Spaces,
                    // mit der Transaktion
                    for (int j = 0; j < targetSpaces.length; j++) {
                        System.out.println("[SpaceRelay.relayMessages()]Schreiben in Space "+targetNames[j]);
                    	targetSpaces[j].write(msg, txn, Lease.FOREVER);
                    	System.out.println("[SpaceRelay.relayMessages()]Message " + i + " wurde in " + targetNames[j]+" eingetragen.");
                    }

                } catch (Exception e) {
                    System.err.println("\n[SpaceRelay.relayMessages()]Message " + i + " kann nicht geschrieben werden: " + e);
                    txn.abort();
                    return;
                }
                txn.commit();
                System.out.println("[SpaceRelay.relayMessages()]Committen der Transaktion " + i);

            } catch (Exception e) {
                System.err.println("[SpaceRelay.relayMessages()]Die Transaktion schlug fehl");
                return;
            }
        }
                    // was ist jetzt noch im Source Space?
                    try {
                      k=0;
                      System.out.println("\n\n[SpaceRelay.relayMessages()]Abfrage derMessages im Source Space");
                      while(true) {
                        msg=(Message)sourceSpace.take(template,null,1000);
                        System.out.println("[SpaceRelay.relayMessages()]Folgende Message wurde aus dem Source Space gelesen:"+msg.content);
                        k++;
                      }
                    } catch(Exception ex) {System.out.println("[SpaceRelay.relayMessages()]Im Source Space befinden sich "+k+" Messages");}

                    // was ist jetzt noch in den Target Space?
                    for (int l=0; l<targetNames.length; l++)
                    try {
                      k=0;
                      System.out.println("[SpaceRelay.relayMessages()]Abfrage derMessages im Target Space"+targetNames[l]);
                      while(true) {
                        msg=(Message)targetSpaces[l].take(template,null,1000);
                        System.out.println("[SpaceRelay.relayMessages()]Folgende Message wurde aus dem Target Space "+targetNames[l]+" gelesen:"+msg.content);
                        k++;
                      }
                    } catch(Exception ex) {System.out.println("[SpaceRelay.relayMessages()]Im Target Space "+targetNames[l]+" befinden sich "+k+" Messages");}
    }

    // Konstruktor
    public SpaceRelay(String[] args) throws IOException {

        // lies den Namen des Source Spaces
        // des  Target Spaces und die ANzahl Messages
        sourceName = args[0];
        System.out.println("[SpaceRelay.relayMessages()]SourceSpace="+sourceName);
        targetNames = new String[args.length - 2];
        for (int i = 0; i < args.length - 2; i++) {
            targetNames[i] = args[i+1];
            System.out.println("[SpaceRelay.relayMessages()]TargetSpace["+i+"]="+targetNames[i]);
        }
        numMessages = Integer.parseInt(args[args.length - 1]);

        // Zugriff auf Source und Target erstellen
        sourceSpace = SpaceAccessor.getSpace(sourceName);
        targetSpaces = new JavaSpace[targetNames.length];
        for (int i = 0; i < targetNames.length; i++) {
            targetSpaces[i] = SpaceAccessor.getSpace(targetNames[i]);
        }

        // kreiere die Messages und spiechere diese im Source Space
        createMessages();

        // relay Messages vom Source ins Target Space
        relayMessages();
    }


    public static void main(String args[]) {
        if (args.length < 3) {
            System.out.println("Usage: SpaceRelay source target1 target2 ... targetN numMessages");
            System.exit(1);
        }

        try {
            SpaceRelay spaceRelay = new SpaceRelay(args);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}