package javaspacesverteilterrechner;

import net.jini.core.entry.*;
import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;

/**
 * Title:        Distributed Computing
 * Description:  einfacher Rechner, welcher JavaSpaces als Kommunikationsplattform, Objektrepository fr die Kommunikation zwischen Clients und dem Master / Server verwendet.
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Master {
    private JavaSpace space;

    public static void main(String[] args) {
        System.out.println("[Master]main() Start");
        Master master = new Master();
        master.startComputing();
    }

    private void startComputing() {
        System.out.println("[Master]startComputing()");
        space = SpaceAccessor.getSpace();

        generateTasks();
        collectResults();
    }

    private void generateTasks() {
        System.out.println("[Master]generateTask()");
        for (int i = 0; i < 10; i++) {
            writeTask(new AddTask(new Integer(i), new Integer(i)));
        }
        for (int i = 0; i < 10; i++) {
            writeTask(new MultTask(new Integer(i), new Integer(i)));
        }
    }

    private void collectResults() {
        System.out.println("[Master]collectResults()");
        for (int i = 0; i < 20; i++) {
            ResultEntry result = takeResult();
            if (result != null) {
                System.out.println(result);
            }
        }
    }

    private void writeTask(Command task) {
        System.out.println("[Master]writeTask");
        try {
            space.write(task, null, Lease.FOREVER);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected ResultEntry takeResult() {
        System.out.println("[Master]takeResult()");
        ResultEntry template = new ResultEntry();
        try {
            ResultEntry result = (ResultEntry)
                space.take(template, null, Long.MAX_VALUE);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}

